source("[..ADAPT DEPENDING ON WHERE YOU UNPACKED THIS SUPPLEMENTARY MATERIAL..]/Function_to_Simulate_evolution_of_symbiont_communities.R")
install.packages('extraDistr')
install.packages('gtools')

#######
# The function SimulateEvolutionOfSymbiontCommunities is devised to be usable for theoritical simulation or for data analysis in the ABC framwork.
# The function is made to be rather easy and flexible to use instead of being rather fast to run.
# Feel free to optimze it for the particular purpose you want or to make it more flexible (e.g. community dependant rates of horizontal transmission or maternal transmission faillure)
#######

l=0 ; ok = FALSE ; while(!ok){
Evolution = SimulateEvolutionOfSymbiontCommunities( pop = c('00'=0.1,'10'=0.4,'01'=0.4,'11'=0.1)*1e4
                                                   ,Ngenerations = 1000,SavePopAtGenerations = seq(0,1000,10)
                                                   ,MT = c(0.999,0.999),HT = c(0,0)
                                                   ,SetFitnessesToMaintainPolymorphism = TRUE
                                                    )
ok=Evolution[nrow(Evolution),'00']+Evolution[nrow(Evolution),'11'] > 2*Evolution[nrow(Evolution),'10']+Evolution[nrow(Evolution),'01'] |
   Evolution[nrow(Evolution),'00']+Evolution[nrow(Evolution),'11']*2 < Evolution[nrow(Evolution),'10']+Evolution[nrow(Evolution),'01']
}
matplot(x = as.numeric(substr(rownames(Evolution),2,nchar(rownames(Evolution)))),y = Evolution/sum(Evolution[1,]),type='l',xlab='Generation',ylab='Frequency',ylim=c(0,0.7))
legend(x = 500,0.45,legend = colnames(Evolution),col=1:ncol(Evolution),lty=1:ncol(Evolution))



## Two exemples of uses :
# 1) Setting pop (an obvious example of interaction on the symbiont fitness)
Evolution = SimulateEvolutionOfSymbiontCommunities( pop = c('00'=0.1,'10'=0.4,'01'=0.4,'11'=0.1)*2e3
                                                   ,Ngenerations = 1000,SavePopAtGenerations = seq(0,1000,10)
                                                   ,MT = c(0.96,0.96),HT = c(0.01,0.02)
                                                   ,W = c('00'=0.91,'10'=0.75,'01'=0.75,'11'=1))

matplot(x = as.numeric(substr(rownames(Evolution),2,nchar(rownames(Evolution)))),y = Evolution/sum(Evolution[1,]),type='l',xlab='Generation',ylab='Frequency',ylim=c(0,0.7))
legend(x = 500,0.45,legend = colnames(Evolution),col=1:ncol(Evolution),lty=1:ncol(Evolution))

# 2) Randomly setting the initial population and setting the fitnesses to maintain polymorphism
Evolution = SimulateEvolutionOfSymbiontCommunities(Nsymbionts = 2
                                                   ,NeF = 1e3
                                                   ,SetFitnessesToMaintainPolymorphism = TRUE
                                                   ,Ngenerations = 1e3,SavePopAtGenerations = seq(0,1e3,10)
                                                   ,MT = c(0.9999,0.9995),HT = c(0.0001,0.0002))

Evolution = Evolution[!is.na(Evolution[,1]),]
matplot(x = as.numeric(substr(rownames(Evolution),2,nchar(rownames(Evolution)))),y = Evolution/sum(Evolution[1,]),type='l',xlab='Generation',ylab='Frequency',ylim = c(0,max(Evolution/sum(Evolution[1,]))*1.1))
legend(x = 0,max(Evolution/sum(Evolution[1,]))*1.1,legend = colnames(Evolution),col=1:ncol(Evolution),lty=1:ncol(Evolution))

## simulate 1 field sample at each saved generations : 
sampleSize = 30
SimulatedFieldSamples = 
  t(apply(Evolution, 1, function(pop)
    extraDistr::rmvhyper(nn=1, n = pop, k=sampleSize)[1,]  # simulate sampling without replacement
  ))
colnames(SimulatedFieldSamples) =colnames(Evolution)

## Then you can simulate the behavior of any statistical test you want
# example with the chi²:
pop = colnames(SimulatedFieldSamples) ; matrix(pop,2,2)
# => matrix(...,2,2) is OK
P.values = apply(SimulatedFieldSamples,1,function(pop)chisq.test(matrix(pop,2,2))$p.value)
Tot=sum(Evolution[1,])
Evolution[P.values>0.05,] = NA
matpoints(x = as.numeric(substr(rownames(Evolution),2,nchar(rownames(Evolution)))),y = Evolution/Tot,type='l',lwd=4,lty = 1)
Evolution[which(P.values<0.05),]
legend(x = 400*nrow(Evolution)/100,y = max(Evolution/Tot*1.1,na.rm = TRUE),legend = colnames(Evolution),col=1:ncol(Evolution),lty=1:ncol(Evolution),lwd=4)
text(540*nrow(Evolution)/100,y = max(Evolution/Tot*1.1,na.rm = TRUE)-0.05,labels = 'Generations where the test was \n significant although their was not\ninteraction between symbionts',adj = 0,font = 2)




## EasyABC::ABC_... functions want as imput the 'model' to be a function that recieve a vector of arguments.
## (see for example 'ABC_rejection')
## You should use the following trick : 
MyModel = function(VectorOfArguments){ # Arguments contains the parameters to be estimated by ABC
  SimulateEvolutionOfSymbiontCommunities(
    # Set the arguments by refering to the VectorOfArguments whenever you need
  )
}

# Then use something like
EasyABC::ABC_...(model=MyModel,
                #... any other things you need
                 )

